@@ -1,4 +1,5 @@
 #include "libmpeg3.h"
+#include "mpeg3private.h"
 #include "mpeg3protos.h"
 
 #include <fcntl.h>
@@ -7,13 +8,32 @@
 
 #define MAX(a, b) ((a) > (b) ? (a) : (b))
 
+int mpeg3_major()
+{
+	return MPEG3_MAJOR;
+}
+
+int mpeg3_minor()
+{
+	return MPEG3_MINOR;
+}
+
+int mpeg3_release()
+{
+	return MPEG3_RELEASE;
+}
+
+
+
 mpeg3_t* mpeg3_new(char *path)
 {
 	int i;
 	mpeg3_t *file = calloc(1, sizeof(mpeg3_t));
 	file->cpus = 1;
 	file->fs = mpeg3_new_fs(path);
-	file->have_mmx = mpeg3_mmx_test();
+//	file->have_mmx = mpeg3_mmx_test();
+// Late compilers don't produce usable code.
+	file->have_mmx = 0;
 	file->demuxer = mpeg3_new_demuxer(file, 0, 0, -1);
 	file->seekable = 1;
 	return file;
@@ -54,6 +74,8 @@ int mpeg3_delete(mpeg3_t *file)
 		free(file->sample_offsets);
 		free(file->total_sample_offsets);
 	}
+	if(file->channel_counts)
+		free(file->channel_counts);
 
 	free(file);
 	return 0;
@@ -174,20 +196,34 @@ static uint64_t read_int64(unsigned char *buffer, int *position)
 
 
 
-static int read_toc(mpeg3_t *file)
+static int read_toc(mpeg3_t *file, int *atracks_return, int *vtracks_return)
 {
 	unsigned char *buffer;
 	int file_type;
 	int position = 4;
 	int stream_type;
-	int atracks;
-	int vtracks;
 	int i, j;
+	int is_vfs = 0;
+	int vfs_len = strlen(RENDERFARM_FS_PREFIX);
+	int toc_version;
+	int64_t current_byte = 0;
+
+// Fix title paths for Cinelerra VFS
+	if(!strncmp(file->fs->path, RENDERFARM_FS_PREFIX, vfs_len))
+		is_vfs = 1;
 
 	buffer = malloc(mpeg3io_total_bytes(file->fs));
 	mpeg3io_seek(file->fs, 0);
 	mpeg3io_read_data(buffer, mpeg3io_total_bytes(file->fs), file->fs);
 
+// Test version
+	if((toc_version = buffer[position++]) != MPEG3_TOC_VERSION)
+	{
+		fprintf(stderr,
+			"read_toc: invalid TOC version %x\n", 
+			toc_version);
+		return 1;
+	}
 
 //printf("read_toc %lld\n", mpeg3io_total_bytes(file->fs));
 
@@ -209,6 +245,7 @@ static int read_toc(mpeg3_t *file)
 			break;
 	}
 
+//printf("read_toc 10\n");
 
 // Stream ID's
 	while((stream_type = buffer[position]) != TITLE_PATH)
@@ -234,6 +271,7 @@ static int read_toc(mpeg3_t *file)
 
 
 
+//printf("read_toc 10\n");
 
 
 // Titles
@@ -242,50 +280,123 @@ static int read_toc(mpeg3_t *file)
 		char string[MPEG3_STRLEN];
 		int string_len = 0;
 		mpeg3_title_t *title;
+		FILE *test_fd;
 
+// Construct title path from VFS prefix and path.
 		position++;
+		if(is_vfs)
+		{
+			strcpy(string, RENDERFARM_FS_PREFIX);
+			string_len = vfs_len;
+		}
 		while(buffer[position] != 0) string[string_len++] = buffer[position++];
 		string[string_len++] = 0;
 		position++;
 
+// Test title availability
+		test_fd = fopen(string, "r");
+		if(test_fd) 
+		{
+			fclose(test_fd);
+		}
+		else
+		{
+// Try concatenating title and toc directory if title is not absolute and
+// toc path has a directory section.
+			if((!is_vfs && string[0] != '/') ||
+				(is_vfs && string[vfs_len] != '/'))
+			{
+// Get toc filename without path
+				char *ptr = strrchr(file->fs->path, '/');
+				if(ptr)
+				{
+					char string2[MPEG3_STRLEN];
+
+// Stack filename on toc path
+					strcpy(string2, file->fs->path);
+					if(!is_vfs)
+						strcpy(&string2[ptr - file->fs->path + 1], string);
+					else
+						strcpy(&string2[ptr - file->fs->path + 1], string + vfs_len);
+
+					test_fd = fopen(string2, "r");
+					if(test_fd)
+					{
+						fclose(test_fd);
+						strcpy(string, string2);
+					}
+					else
+					{
+						fprintf(stderr, 
+							"read_toc: failed to open %s or %s\n",
+							string,
+							string2);
+						return 1;
+					}
+				}
+				else
+				{
+					fprintf(stderr,
+						"read_toc: failed to open %s\n", 
+						string);
+					return 1;
+				}
+			}
+			else
+			{
+				fprintf(stderr, 
+					"read_toc: failed to open %s\n", 
+					string);
+				return 1;
+			}
+		}
+
 		title = 
 			file->demuxer->titles[file->demuxer->total_titles++] = 
 			mpeg3_new_title(file, string);
 
 		title->total_bytes = read_int64(buffer, &position);
+		title->start_byte = current_byte;
+		title->end_byte = title->start_byte + title->total_bytes;
+		current_byte = title->end_byte;
 
 // Cells
-		title->timecode_table_size = 
-			title->timecode_table_allocation = 
+		title->cell_table_size = 
+			title->cell_table_allocation = 
 			read_int32(buffer, &position);
-		title->timecode_table = calloc(title->timecode_table_size, sizeof(mpeg3demux_timecode_t));
-		for(i = 0; i < title->timecode_table_size; i++)
+		title->cell_table = calloc(title->cell_table_size, sizeof(mpeg3demux_cell_t));
+		for(i = 0; i < title->cell_table_size; i++)
 		{
-			title->timecode_table[i].start_byte = read_int64(buffer, &position);
-			title->timecode_table[i].end_byte = read_int64(buffer, &position);
-			title->timecode_table[i].program = read_int32(buffer, &position);
+			title->cell_table[i].start_byte = read_int64(buffer, &position);
+			title->cell_table[i].end_byte = read_int64(buffer, &position);
+			title->cell_table[i].program = read_int32(buffer, &position);
 		}
 	}
+//printf("read_toc 10\n");
 
 
 
 // Audio streams
 // Skip ATRACK_COUNT
 	position++;
-	atracks = read_int32(buffer, &position);
+	*atracks_return = read_int32(buffer, &position);
+//printf("read_toc 10\n");
 
 // Skip VTRACK_COUNT
 	position++;
-	vtracks = read_int32(buffer, &position);
+	*vtracks_return = read_int32(buffer, &position);
+//printf("read_toc 10\n");
 
 
-	if(atracks)
+	if(*atracks_return)
 	{
-		file->sample_offsets = malloc(sizeof(int64_t*) * atracks);
-		file->total_sample_offsets = malloc(sizeof(int*) * atracks);
+		file->channel_counts = calloc(sizeof(int), *atracks_return);
+		file->sample_offsets = malloc(sizeof(int64_t*) * *atracks_return);
+		file->total_sample_offsets = malloc(sizeof(int*) * *atracks_return);
 
-		for(i = 0; i < atracks; i++)
+		for(i = 0; i < *atracks_return; i++)
 		{
+			file->channel_counts[i] = read_int32(buffer, &position);
 			file->total_sample_offsets[i] = read_int32(buffer, &position);
 			file->sample_offsets[i] = malloc(file->total_sample_offsets[i] * sizeof(int64_t));
 			for(j = 0; j < file->total_sample_offsets[i]; j++)
@@ -295,15 +406,16 @@ static int read_toc(mpeg3_t *file)
 			}
 		}
 	}
+//printf("read_toc 10\n");
 
-	if(vtracks)
+	if(*vtracks_return)
 	{
-		file->frame_offsets = malloc(sizeof(int64_t*) * vtracks);
-		file->total_frame_offsets = malloc(sizeof(int*) * vtracks);
-		file->keyframe_numbers = malloc(sizeof(int64_t*) * vtracks);
-		file->total_keyframe_numbers = malloc(sizeof(int*) * vtracks);
+		file->frame_offsets = malloc(sizeof(int64_t*) * *vtracks_return);
+		file->total_frame_offsets = malloc(sizeof(int*) * *vtracks_return);
+		file->keyframe_numbers = malloc(sizeof(int64_t*) * *vtracks_return);
+		file->total_keyframe_numbers = malloc(sizeof(int*) * *vtracks_return);
 
-		for(i = 0; i < vtracks; i++)
+		for(i = 0; i < *vtracks_return; i++)
 		{
 			file->total_frame_offsets[i] = read_int32(buffer, &position);
 			file->frame_offsets[i] = malloc(file->total_frame_offsets[i] * sizeof(int64_t));
@@ -322,13 +434,16 @@ static int read_toc(mpeg3_t *file)
 			}
 		}
 	}
+//printf("read_toc 10\n");
 
 	free(buffer);
+//printf("read_toc 10\n");
 
 
 
 //printf("read_toc 1\n");
 	mpeg3demux_open_title(file->demuxer, 0);
+//printf("read_toc 10\n");
 
 //printf("read_toc 2 %llx\n", mpeg3demux_tell(file->demuxer));
 	return 0;
@@ -342,6 +457,10 @@ mpeg3_t* mpeg3_open_copy(char *path, mpeg3_t *old_file)
 	mpeg3_t *file = 0;
 	unsigned int bits;
 	int i, done;
+/* The table of contents may have fewer tracks than are in the demuxer */
+/* This limits the track count */
+	int toc_atracks = 0x7fffffff;
+	int toc_vtracks = 0x7fffffff;
 
 /* Initialize the file structure */
 	file = mpeg3_new(path);
@@ -352,7 +471,7 @@ mpeg3_t* mpeg3_open_copy(char *path, mpeg3_t *old_file)
 
 
 
-//printf("mpeg3_open_copy %s\n", path);
+//printf("mpeg3_open_copy 1 %s\n", path);
 
 /* Need to perform authentication before reading a single byte. */
 	if(mpeg3io_open_file(file->fs))
@@ -382,15 +501,12 @@ mpeg3_t* mpeg3_open_copy(char *path, mpeg3_t *old_file)
 /* Table of contents for another title set */
 		if(!old_file)
 		{
-//printf("libmpeg3 1\n");
-			if(read_toc(file))
+			if(read_toc(file, &toc_atracks, &toc_vtracks))
 			{
-//printf("libmpeg3 1\n");
 				mpeg3io_close_file(file->fs);
 				mpeg3_delete(file);
 				return 0;
 			}
-//printf("libmpeg3 1\n");
 		}
 		mpeg3io_close_file(file->fs);
 	}
@@ -398,17 +514,14 @@ mpeg3_t* mpeg3_open_copy(char *path, mpeg3_t *old_file)
 // IFO file
     if(bits == MPEG3_IFO_PREFIX)
     {
-//printf("libmpeg3 1\n");
     	if(!old_file)
 		{
-//printf("libmpeg3 2\n");
 			if(mpeg3_read_ifo(file, 0))
         	{
 				mpeg3_delete(file);
 				mpeg3io_close_file(file->fs);
 				return 0;
         	}
-//printf("libmpeg3 3\n");
 		}
 		file->is_ifo_file = 1;
 		mpeg3io_close_file(file->fs);
@@ -417,7 +530,6 @@ mpeg3_t* mpeg3_open_copy(char *path, mpeg3_t *old_file)
 	if(((bits >> 24) & 0xff) == MPEG3_SYNC_BYTE)
 	{
 /* Transport stream */
-//printf("libmpeg3 2\n");
 		file->is_transport_stream = 1;
 	}
 	else
@@ -425,7 +537,6 @@ mpeg3_t* mpeg3_open_copy(char *path, mpeg3_t *old_file)
 	{
 /* Program stream */
 /* Determine packet size empirically */
-//printf("libmpeg3 3\n");
 		file->is_program_stream = 1;
 	}
 	else
@@ -435,7 +546,6 @@ mpeg3_t* mpeg3_open_copy(char *path, mpeg3_t *old_file)
 		(bits == MPEG3_RIFF_CODE))
 	{
 /* MPEG Audio only */
-//printf("libmpeg3 4\n");
 		file->is_audio_stream = 1;
 	}
 	else
@@ -443,27 +553,29 @@ mpeg3_t* mpeg3_open_copy(char *path, mpeg3_t *old_file)
 		bits == MPEG3_PICTURE_START_CODE)
 	{
 /* Video only */
-//printf("libmpeg3 5\n");
 		file->is_video_stream = 1;
 	}
 	else
 	if(((bits & 0xffff0000) >> 16) == MPEG3_AC3_START_CODE)
 	{
 /* AC3 Audio only */
-//printf("libmpeg3 6\n");
 		file->is_audio_stream = 1;
 	}
 	else
 	{
-//printf("libmpeg3 7\n");
 		mpeg3_delete(file);
 		fprintf(stderr, "mpeg3_open: not an MPEG 2 stream\n");
 		return 0;
 	}
 
-//printf("mpeg3_open 2 %p %d %d %d %d\n", 
-//	old_file, file->is_transport_stream, file->is_program_stream, file->is_video_stream, file->is_audio_stream);
-
+/*
+ * printf("mpeg3_open 2 %p %d %d %d %d\n", 
+ * old_file, 
+ * file->is_transport_stream, 
+ * file->is_program_stream, 
+ * file->is_video_stream, 
+ * file->is_audio_stream);
+ */
 
 
 
@@ -520,7 +632,7 @@ mpeg3_t* mpeg3_open_copy(char *path, mpeg3_t *old_file)
 
 
 
-//printf("mpeg3_open 2\n");
+//printf("mpeg3_open 50\n");
 
 
 
@@ -536,9 +648,9 @@ mpeg3_t* mpeg3_open_copy(char *path, mpeg3_t *old_file)
 	if(file->is_transport_stream || file->is_program_stream)
 	{
 /* Create video tracks */
-/* Video must be created before audio because audio uses the video timecode */
-/* to get its length. */
-		for(i = 0; i < MPEG3_MAX_STREAMS; i++)
+		for(i = 0; 
+			i < MPEG3_MAX_STREAMS && file->total_vstreams < toc_vtracks; 
+			i++)
 		{
 			if(file->demuxer->vstream_table[i])
 			{
@@ -550,12 +662,11 @@ mpeg3_t* mpeg3_open_copy(char *path, mpeg3_t *old_file)
 				if(file->vtrack[file->total_vstreams]) 
 					file->total_vstreams++;
 
-//printf("libmpeg3 %d %d %p %d\n", i, file->demuxer->vstream_table[i], file->vtrack[file->total_vstreams], file->total_vstreams);
 			}
 		}
 
 /* Create audio tracks */
-		for(i = 0; i < MPEG3_MAX_STREAMS; i++)
+		for(i = 0; i < MPEG3_MAX_STREAMS && file->total_astreams < toc_atracks; i++)
 		{
 			if(file->demuxer->astream_table[i])
 			{
@@ -572,12 +683,10 @@ mpeg3_t* mpeg3_open_copy(char *path, mpeg3_t *old_file)
 	if(file->is_video_stream)
 	{
 /* Create video tracks */
-//printf("mpeg3_open 3\n");
 		file->vtrack[0] = mpeg3_new_vtrack(file, 
 			-1, 
 			file->demuxer, 
 			0);
-//printf("mpeg3_open 4\n");
 		if(file->vtrack[0]) file->total_vstreams++;
 	}
 	else
@@ -585,17 +694,14 @@ mpeg3_t* mpeg3_open_copy(char *path, mpeg3_t *old_file)
 	{
 /* Create audio tracks */
 
-//printf("mpeg3_open 3\n");
 		file->atrack[0] = mpeg3_new_atrack(file, 
 			-1, 
 			AUDIO_UNKNOWN, 
 			file->demuxer,
 			0);
-//printf("mpeg3_open 3\n");
 		if(file->atrack[0]) file->total_astreams++;
 	}
 
-//printf("mpeg3_open 5\n");
 
 
 
@@ -673,6 +779,7 @@ int mpeg3_set_sample(mpeg3_t *file,
 {
 	if(file->total_astreams)
 	{
+//printf(__FUNCTION__ " 1 %d %d\n", file->atrack[stream]->current_position, sample);
 		file->atrack[stream]->current_position = sample;
 		mpeg3audio_seek_sample(file->atrack[stream]->audio, sample);
 		return 0;
@@ -738,7 +845,7 @@ float mpeg3_aspect_ratio(mpeg3_t *file, int stream)
 	return 0;
 }
 
-float mpeg3_frame_rate(mpeg3_t *file,
+double mpeg3_frame_rate(mpeg3_t *file,
 		int stream)
 {
 	if(file->total_vstreams)
@@ -775,22 +882,38 @@ int mpeg3_set_frame(mpeg3_t *file,
 	return -1;
 }
 
-int mpeg3_seek_percentage(mpeg3_t *file, double percentage)
+int mpeg3_seek_byte(mpeg3_t *file, int64_t byte)
 {
 	int i;
+
+//	file->percentage_pts = -1;
 	for(i = 0; i < file->total_vstreams; i++)
 	{
-		mpeg3video_seek_percentage(file->vtrack[i]->video, percentage);
+		file->vtrack[i]->current_position = 0;
+		mpeg3video_seek_byte(file->vtrack[i]->video, byte);
 	}
 
 	for(i = 0; i < file->total_astreams; i++)
 	{
-		mpeg3audio_seek_percentage(file->atrack[i]->audio, percentage);
+		file->atrack[i]->current_position = 0;
+		mpeg3audio_seek_byte(file->atrack[i]->audio, byte);
 	}
 
 	return 0;
 }
 
+/*
+ * double mpeg3_get_percentage_pts(mpeg3_t *file)
+ * {
+ * 	return file->percentage_pts;
+ * }
+ * 
+ * void mpeg3_set_percentage_pts(mpeg3_t *file, double pts)
+ * {
+ * }
+ */
+
+
 int mpeg3_previous_frame(mpeg3_t *file, int stream)
 {
 	file->last_type_read = 2;
@@ -802,19 +925,24 @@ int mpeg3_previous_frame(mpeg3_t *file, int stream)
 	return 0;
 }
 
-double mpeg3_tell_percentage(mpeg3_t *file)
+int64_t mpeg3_tell_byte(mpeg3_t *file)
 {
-	double percent = 0;
+	int64_t result = 0;
 	if(file->last_type_read == 1)
 	{
-		percent = mpeg3demux_tell_percentage(file->atrack[file->last_stream_read]->demuxer);
+		result = mpeg3demux_tell_byte(file->atrack[file->last_stream_read]->demuxer);
 	}
 
 	if(file->last_type_read == 2)
 	{
-		percent = mpeg3demux_tell_percentage(file->vtrack[file->last_stream_read]->demuxer);
+		result = mpeg3demux_tell_byte(file->vtrack[file->last_stream_read]->demuxer);
 	}
-	return percent;
+	return result;
+}
+
+int64_t mpeg3_get_bytes(mpeg3_t *file)
+{
+	return mpeg3demux_movie_size(file->demuxer);
 }
 
 double mpeg3_get_time(mpeg3_t *file)
@@ -839,14 +967,18 @@ double mpeg3_get_time(mpeg3_t *file)
 /* Use percentage and total time */
 		if(file->total_astreams)
 		{
-			atime = mpeg3demux_tell_percentage(file->atrack[0]->demuxer) * 
-						mpeg3_audio_samples(file, 0) / mpeg3_sample_rate(file, 0);
+			atime = mpeg3demux_tell_byte(file->atrack[0]->demuxer) * 
+						mpeg3_audio_samples(file, 0) / 
+						mpeg3_sample_rate(file, 0) /
+						mpeg3_get_bytes(file);
 		}
 
 		if(file->total_vstreams)
 		{
-			vtime = mpeg3demux_tell_percentage(file->vtrack[0]->demuxer) *
-						mpeg3_video_frames(file, 0) / mpeg3_frame_rate(file, 0);
+			vtime = mpeg3demux_tell_byte(file->vtrack[0]->demuxer) *
+						mpeg3_video_frames(file, 0) / 
+						mpeg3_frame_rate(file, 0) /
+						mpeg3_get_bytes(file);
 		}
 	}
 
@@ -856,6 +988,7 @@ double mpeg3_get_time(mpeg3_t *file)
 int mpeg3_end_of_audio(mpeg3_t *file, int stream)
 {
 	int result = 0;
+	if(!file->atrack[stream]->channels) return 1;
 	result = mpeg3demux_eof(file->atrack[stream]->demuxer);
 	return result;
 }
@@ -868,38 +1001,6 @@ int mpeg3_end_of_video(mpeg3_t *file, int stream)
 }
 
 
-int mpeg3_read_frame(mpeg3_t *file, 
-		unsigned char **output_rows, 
-		int in_x, 
-		int in_y, 
-		int in_w, 
-		int in_h, 
-		int out_w, 
-		int out_h, 
-		int color_model,
-		int stream)
-{
-	int result = -1;
-
-	if(file->total_vstreams)
-	{
-		result = mpeg3video_read_frame(file->vtrack[stream]->video, 
-					file->vtrack[stream]->current_position, 
-					output_rows,
-					in_x, 
-					in_y, 
-					in_w, 
-					in_h, 
-					out_w,
-					out_h,
-					color_model);
-		file->last_type_read = 2;
-		file->last_stream_read = stream;
-		file->vtrack[stream]->current_position++;
-	}
-	return result;
-}
-
 int mpeg3_drop_frames(mpeg3_t *file, long frames, int stream)
 {
 	int result = -1;
@@ -934,6 +1035,42 @@ int mpeg3_set_rowspan(mpeg3_t *file, int bytes, int stream)
 }
 
 
+int mpeg3_read_frame(mpeg3_t *file, 
+		unsigned char **output_rows, 
+		int in_x, 
+		int in_y, 
+		int in_w, 
+		int in_h, 
+		int out_w, 
+		int out_h, 
+		int color_model,
+		int stream)
+{
+	int result = -1;
+//printf("mpeg3_read_frame 1 %d\n", file->vtrack[stream]->current_position);
+
+	if(file->total_vstreams)
+	{
+		result = mpeg3video_read_frame(file->vtrack[stream]->video, 
+					file->vtrack[stream]->current_position, 
+					output_rows,
+					in_x, 
+					in_y, 
+					in_w, 
+					in_h, 
+					out_w,
+					out_h,
+					color_model);
+//printf(__FUNCTION__ " 2\n");
+		file->last_type_read = 2;
+		file->last_stream_read = stream;
+		file->vtrack[stream]->current_position++;
+	}
+
+//printf("mpeg3_read_frame 2 %d\n", file->vtrack[stream]->current_position);
+	return result;
+}
+
 int mpeg3_read_yuvframe(mpeg3_t *file,
 		char *y_output,
 		char *u_output,
@@ -946,7 +1083,7 @@ int mpeg3_read_yuvframe(mpeg3_t *file,
 {
 	int result = -1;
 
-//printf("mpeg3_read_yuvframe 1 %d %d\n", mpeg3demux_tell(file->vtrack[stream]->demuxer), mpeg3demuxer_total_bytes(file->vtrack[stream]->demuxer));
+//printf("mpeg3_read_yuvframe 1\n");
 	if(file->total_vstreams)
 	{
 		result = mpeg3video_read_yuvframe(file->vtrack[stream]->video, 
@@ -962,7 +1099,7 @@ int mpeg3_read_yuvframe(mpeg3_t *file,
 		file->last_stream_read = stream;
 		file->vtrack[stream]->current_position++;
 	}
-//printf("mpeg3_read_yuvframe 2 %d %d\n", mpeg3demux_tell(file->vtrack[stream]->demuxer), mpeg3demuxer_total_bytes(file->vtrack[stream]->demuxer));
+//printf("mpeg3_read_yuvframe 100\n");
 	return result;
 }
 
@@ -974,7 +1111,6 @@ int mpeg3_read_yuvframe_ptr(mpeg3_t *file,
 {
 	int result = -1;
 
-//printf("mpeg3_read_yuvframe 1 %d %d\n", mpeg3demux_tell(file->vtrack[stream]->demuxer), mpeg3demuxer_total_bytes(file->vtrack[stream]->demuxer));
 	if(file->total_vstreams)
 	{
 		result = mpeg3video_read_yuvframe_ptr(file->vtrack[stream]->video, 
@@ -986,7 +1122,6 @@ int mpeg3_read_yuvframe_ptr(mpeg3_t *file,
 		file->last_stream_read = stream;
 		file->vtrack[stream]->current_position++;
 	}
-//printf("mpeg3_read_yuvframe 2 %d %d\n", mpeg3demux_tell(file->vtrack[stream]->demuxer), mpeg3demuxer_total_bytes(file->vtrack[stream]->demuxer));
 	return result;
 }
 
@@ -999,20 +1134,17 @@ int mpeg3_read_audio(mpeg3_t *file,
 {
 	int result = -1;
 
-//printf("mpeg3_read_audio 1 %d %d\n", mpeg3demux_tell(file->atrack[stream]->demuxer), mpeg3demuxer_total_bytes(file->atrack[stream]->demuxer));
 	if(file->total_astreams)
 	{
 		result = mpeg3audio_decode_audio(file->atrack[stream]->audio, 
 					output_f, 
 					output_i, 
 					channel, 
-					file->atrack[stream]->current_position, 
 					samples);
 		file->last_type_read = 1;
 		file->last_stream_read = stream;
 		file->atrack[stream]->current_position += samples;
 	}
-//printf("mpeg3_read_audio 2 %d %d\n", mpeg3demux_tell(file->atrack[stream]->demuxer), mpeg3demuxer_total_bytes(file->atrack[stream]->demuxer));
 
 	return result;
 }
