@@ -1,5 +1,5 @@
-#include "../mpeg3private.h"
-#include "mpeg3video.h"
+#include "mpeg3private.h"
+#include "mpeg3protos.h"
 #include "vlc.h"
 
 #include <stdio.h>
@@ -44,7 +44,8 @@ int mpeg3video_clearblock(mpeg3_slice_t *slice, int comp, int size)
 
 /* Compiler error with 2.95 required hard coding the size to 6 */
 
-	memset(slice->block[comp], 0, sizeof(short) * 64 * size);
+	bzero(slice->block[comp], sizeof(short) * 64 * size);
+//	memset(slice->block[comp], 0, sizeof(short) * 64 * size);
 	return 0;
 }
 
@@ -114,7 +115,7 @@ int mpeg3video_getintrablock(mpeg3_slice_t *slice,
 		int comp, 
 		int dc_dct_pred[])
 {
-	int val, i, j, sign;
+	int val, i, j = 8, sign;
 	unsigned int code;
 	mpeg3_DCTtab_t *tab = 0;
 	short *bp = slice->block[comp];
@@ -714,7 +715,9 @@ int mpeg3video_getpicture(mpeg3video_t *video, int framenum)
     	if(video->pict_struct == BOTTOM_FIELD)
 		{
 /* Only used if fields are in different pictures */
-    	    video->newframe[i] += (i == 0) ? video->coded_picture_width : video->chrom_width;
+    	    video->newframe[i] += (i == 0) ? 
+				video->coded_picture_width : 
+				video->chrom_width;
 		}
 	}
 
@@ -755,6 +758,7 @@ int mpeg3video_getpicture(mpeg3video_t *video, int framenum)
 		video->current_repeat += 100;
 	}
 
-  	if(video->pict_struct != FRAME_PICTURE) video->secondfield = !video->secondfield;
+  	if(video->pict_struct != FRAME_PICTURE) 
+		video->secondfield = !video->secondfield;
 	return result;
 }
